TTYOFF:TRUE;

/* #############################################################################################################
   ### Purpose: Provide a function for numerical integration that 
   ### - avoids the strange behaviour of the built-in Romberg function in the four-argument form. 
   ### - Does not produce an error message, when given nonnumeric values, but returns the noun form
   #############################################################################################################
*/
NumIntegrate(MyExpr,MyVar,MyStart,MyStop):=BLOCK(
 [MyRes],
 CATCH(
  
 /* Romberg throws an error, if non-numeric arguments are tried */
 IF NUMBERP(MyStart) AND NUMBERP(MyStop) THEN
 BLOCK(
   MyRes:ERRCATCH(
    /* ROMBERG is a macro. Therefore use APPLY. */
    apply('ROMBERG,[MyExpr,MyVar,MyStart,MyStop])
    ) /*errcatch*/,
   IF MyRes # [] THEN 
   /* In case of success, return the result */
   THROW(MyRes[1])
   ), /* Block if Start/stop numeric*/
  
    /* In any other case, return the noun form (unchanged)*/
 THROW(
  FUNMAKE('NumIntegrate,[MyExpr,MyVar,MyStart,MyStop])
  ) /*Throw Noun-form*/
 ) /*Catch*/
)$/* Block NumIntegrate */
TTYOFF:FALSE$

if SYMAXX_SELFTEST = TRUE then
/* Check a numeric result*/
block([Result],
 Result:NumIntegrate(x,x,0,1),
 if Result=0.5 then print("OK")
 else SYMAXX_SELFTEST:"Failed at NumIntegrate Test 1"
)$ /* Selftest block*/	

if SYMAXX_SELFTEST = TRUE then
/* Check noun form for a nonnumeric start */
block([Result],
 Result:NumIntegrate(x,x,a,1),
 if Result=NumIntegrate(x,x,a,1) then print("OK")
 else SYMAXX_SELFTEST:"Failed at NumIntegrate Test 2"
)$ /* Selftest block*/	

if SYMAXX_SELFTEST = TRUE then
/* Check noun form for a nonnumeric stop */
block([Result],
 Result:NumIntegrate(x,x,a,1),
 if Result=NumIntegrate(x,x,a,1) then print("OK")
 else SYMAXX_SELFTEST:"Failed at NumIntegrate Test 3"
)$ /* Selftest block*/	

if SYMAXX_SELFTEST = TRUE then
/* Check noun form for a nonnumeric-evaluating expression */
block([Result],
 Result:NumIntegrate(a*x,x,0,1),
 if Result=NumIntegrate(a*x,x,0,1) then print("OK")
 else SYMAXX_SELFTEST:"Failed at NumIntegrate Test 4"
)$ /* Selftest block*/	


/* #############################################################################################################
   ### LinInterp: linear interpolation through a list of points.                                             ###
   ### Example: AlmostSquare(x):=LinInterp(x,[[0,0],[1,1],[2,4],[3,9],[4,16]])                               ###
   #############################################################################################################
*/

LinInterp(x,Pts):=BLOCK(
	[a,First_a:-1,Last_a:-1,deb:""],
	x:ev(x,numer), 
	/* Nothing to do?*/
	if not(numberp(x)) then return(funmake(LinInterp,[x,Pts])),
	FOR a THRU LENGTH(Pts) DO 
		BLOCK([StepX:Pts[a][1]],
		deb:SCONCAT(deb," ",a,StepX,x),
		IF x >= StepX THEN 
			First_a:a,
		IF x < StepX AND Last_a < 0 AND First_a > 0 THEN 
			Last_a:a
		),
	IF First_a < 0 THEN 
		BLOCK(First_a:1,Last_a:1),
		IF Last_a < 0 THEN 
			BLOCK(First_a:LENGTH(Pts),
			Last_a:LENGTH(Pts)
			),
			IF First_a = Last_a THEN 
				Pts[First_a][2] 
			ELSE 
				BLOCK([X1:Pts[First_a][1],X2:Pts[Last_a][1],Y1:Pts[First_a][2],Y2:Pts[Last_a][2]],
				Y1+(Y2-Y1)/(X2-X1)*(x-X1)
				)
)$/*LinInterp block*/

/* #############################################################################################################
   ### Logarithm to the base of 10                                                                           ###
   ### Any scientific pocket calculator has it. Symaxx has it, too.                                          ###
   #############################################################################################################
*/
log10(x):=log(x)/log(10)$

/* #############################################################################################################
   ### Numerical derivative                                                                                  ###
   ### Syntax: NumDiff(Expression, Variable, Value)                                                          ###
   ### Set NumDiffDelta to a suitable value.                                                                 ###
   #############################################################################################################
*/

NumDiffDelta:1.0e-4$
NumDiff(MyExpr,MyVar,MyVar0):=BLOCK(
 [y2:ev(MyExpr,ev(MyVar) = MyVar0+NumDiffDelta/2,numer),
 y1:ev(MyExpr,ev(MyVar) = MyVar0-NumDiffDelta/2,numer)],
 IF NOT NUMBERP(NumDiffDelta) THEN ERROR("Set NumDiffDelta to a small value before using NumDiff()!"),
 CATCH(
  IF NUMBERP(y1) AND NUMBERP(y2) AND NUMBERP(MyVar0) AND NUMBERP(NumDiffDelta) THEN 
  THROW((y2-y1)/NumDiffDelta),
  THROW(FUNMAKE('NumDiff,[MyExpr,MyVar,MyVar0]))
  ) /* Catch */
 )$ /*Block NumDiff*/


/* #############################################################################################################
   ### Table operations                                                                                       ###
   ### Functions to operate on one-dimensional lists                                                         ###
   ### for example: [[0,0],[1,1],[2,4],[3,9],[4,16],...]                                                     ###
   ### is understood as                                                                                      ###
   ### 0 => 0                                                                                                ###
   ### 1 => 1                                                                                                ###
   ### 2 => 4                                                                                                ###
   ### 3 => 9 etc.                                                                                           ###
   #############################################################################################################
*/

/* TableOpInner: Internal TableOp function. Not intended for end user. */
TableOpInner([TableOpInnerArgs]):=BLOCK(
 [TheNrArgs,TheOpArgs,TheOperator,TheIndep],
 TheNrArgs:LENGTH(TableOpInnerArgs)-1,
 TheOperator:LAST(TableOpInnerArgs),
 TheIndep:TableOpInnerArgs[1][1],
 TheOpArgs:APPEND([TheIndep],MAKELIST(TableOpInnerArgs[Count][2],Count,1,TheNrArgs)),
 FOR Count THRU TheNrArgs DO (
  IF TableOpInnerArgs[Count][1] # TheIndep THEN ERROR(CONCAT("Indeps do not match. First is ",TheIndep))
  ),RETURN([TheIndep,APPLY(TheOperator,TheOpArgs)])
 )$

TableOpApply([TableOpApplyArgs]):=CATCH(
 BLOCK(
  [Result:[],DefaultError:"TableOp: First argument must be [[x0,y0],[x1,y1],...],and second argument [[x0,z0],[x1,z1],...]. The x0 must match.",TheColumns:[],TheOperator,TheColumnLength,TheLength],
  BLOCK(
   /*How many columns are supplied?*/
   TheNrColumns:LENGTH(TableOpApplyArgs)-1,
   /*Separate columns and operator*/
   TheColumns:MAKELIST(TableOpApplyArgs[Count],Count,1,TheNrColumns),
   TheOperator:LAST(TableOpApplyArgs),
   /*If the arguments are not lists, then return the noun form.*/
   FOR Count THRU TheNrColumns DO (
    IF NOT LISTP(TheColumns[Count]) THEN THROW(FUNMAKE('TableOpApply,TableOpApplyArgs))
    ),
   /*"Get the length of one column"*/
   TheColumnLength:LENGTH(TheColumns[1]),
   /*"Check, that each column has the same length"*/
   FOR Count THRU TheNrColumns DO (
    IF LENGTH(TheColumns[Count]) # TheColumnLength THEN ERROR(CONCAT(DefaultError,"Column sizes do not match: Col 1 and ",Count))),
   /*"Do the calculation and create the list of results"*/
   FOR Count THRU TheColumnLength DO BLOCK(
    [ArgsForThisCalculation:[],ThisResult],
    /*Pick row 'Count' out of column 'Count2' and put it into ArgsForThisCalculation.
    Apply the operator on ArgsForThisCalculation.
    Append result to 'Results'.*/
    FOR Count2 THRU TheNrColumns DO 
    ArgsForThisCalculation:APPEND(ArgsForThisCalculation,[TheColumns[Count2][Count]]),
    IF ERRCATCH(ThisResult:APPLY('TableOpInner,APPEND(ArgsForThisCalculation,[TheOperator])),1) = [] THEN ERROR(ERROR," at table, line ",Count),
    Result:APPEND(Result,[ThisResult])),
   THROW(Result)
   )
  )
 )$

if SYMAXX_SELFTEST = TRUE then
/* Check noun form for a nonnumeric-evaluating expression */
block([Result],
 Result:TableOpApply(List1,[[-30,a0],[-29,b0],[-28,1]],Lambda([Indep,var1,var2],var1/var2)),
 if Result=TableOpApply(List1,[[-30,a0],[-29,b0],[-28,1]],Lambda([Indep,var1,var2],var1/var2)) then print("OKK")
 else SYMAXX_SELFTEST:"Failed at TableOpApply Test 1"
)$ /* Selftest block*/	
if SYMAXX_SELFTEST = TRUE then
/* Check TableOpApply function */
block([Result],
 Result:TableOpApply([[-30,a],[-29,b],[-28,c]],[[-30,a0],[-29,b0],[-28,1]],Lambda([Indep,var1,var2],var1/var2)),
 if Result=[[-30,a/a0],[-29,b/b0],[-28,c]] then print("OK")
 else SYMAXX_SELFTEST:"Failed at TableOpApply Test 2"
)$ /* Selftest block*/	
TTYOFF:FALSE;

